/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "globus_common.h"
#include "globus_gsi_authz.h"
#include "globus_i_gsi_authz_bdm_callout.h"
#include "globus_gsi_authz_callout_error.h"
#include "globus_gsi_system_config.h"
#include "gaa.h"
#include "gaa_plugin.h"
#include "gaa_gss_generic.h"
#include "gssapi.h"
#include "esg_gridmap_assist.h"
#include "globus_gss_assist.h"
#include "globus_gridmap_callout_error.h"

#include <stdlib.h>

#ifdef BUILD_DEBUG
int      globus_i_gsi_authz_bdm_callout_debug_level   = 0;
FILE *   globus_i_gsi_authz_bdm_callout_debug_fstream = 0;
#endif /* BUILD_DEBUG */

static config_t s_config;
static int s_config_initialized = 0;

globus_result_t
globus_gsi_authz_bdm_gridmap_callout(
    va_list                             ap)
{
    gss_ctx_id_t                        context;
    char *                              service;
    char *                              desired_identity;
    char *                              identity_buffer;
    unsigned int                        buffer_length;
    globus_result_t                     result = !GLOBUS_SUCCESS;
    int                                 ret = -1;

    context = va_arg(ap, gss_ctx_id_t);
    service = va_arg(ap, char *);
    desired_identity = va_arg(ap, char *);
    identity_buffer = va_arg(ap, char *);
    buffer_length = va_arg(ap, unsigned int);

    if (!s_config_initialized)
    {
        set_security_context(&context);

        memset(&s_config, 0, sizeof(config_t));
        ret = parse_config_file("/etc/grid-security/esg_gridmap_assist.conf", &s_config);
        if (ret == 0)
        {   
            s_config_initialized = 1;
        }
    }

    if (s_config_initialized)
    {
        ret = parse_credential_extension(&s_config);
        if (ret == 0)
        {
            strcpy(identity_buffer, s_config.admin_local_account);
            result = GLOBUS_SUCCESS;
        }
    }

    if (result != GLOBUS_SUCCESS)
    {
        GLOBUS_GRIDMAP_CALLOUT_ERROR(
            result,
            GLOBUS_GRIDMAP_CALLOUT_LOOKUP_FAILED,
            ("User is not a BDM administrator and cannot transfer this data.\n"));
    }

    return result;
}
